#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_skin.interfaces.container import ITableElementName
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces.zeo import IZEOConnection
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent, DISPLAY_MODE
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_utils.registry import query_utility
from pyams_utils.url import absolute_url
from pyams_utils.zodb import ZEOConnectionUtility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.control_panel import UtilitiesTable
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Invalid

from pyams_utils import _


@adapter_config(context=(IZEOConnection, IAdminLayer), provides=ITableElementName)
class ZEOConnectionNameAdapter(ContextRequestAdapter):
    """ZEO connection name adapter"""

    @property
    def name(self):
        translate = self.request.localizer.translate
        return translate(_("ZEO: {0}")).format(self.context.name)


@viewlet_config(name='add-zeo-connection.menu', context=ISite, layer=IAdminLayer,
                view=UtilitiesTable, manager=IToolbarAddingMenu, permission='system.manage')
class ZEOConnectionAddMenu(ToolbarMenuItem):
    """ZEO connection add menu"""

    label = _("Add ZEO connection...")
    label_css_class = 'fa fa-fw fa-database'
    url = 'add-zeo-connection.html'
    modal_target = True


@pagelet_config(name='add-zeo-connection.html', context=ISite, layer=IPyAMSLayer, permission='system.manage')
class ZEOConnectionAddForm(AdminDialogAddForm):
    """ZEO connection add form"""

    title = _("Utilities")
    legend = _("Add ZEO connection")
    icon_css_class = 'fa fa-fw fa-database'

    fields = field.Fields(IZEOConnection)
    autocomplete = 'off'
    ajax_handler = 'add-zeo-connection.json'
    edit_permission = None

    def create(self, data):
        return ZEOConnectionUtility()

    def add(self, object):
        manager = self.context.getSiteManager()
        manager['zeo::{0}'.format(object.name.lower())] = object

    def nextURL(self):
        return absolute_url(self.context, self.request, 'utilities.html')


@subscriber(IDataExtractedEvent, form_selector=ZEOConnectionAddForm)
def handle_new_connection_data_extraction(event):
    """Handle new connection data extraction"""
    manager = event.form.context.getSiteManager()
    name = event.data['name']
    if 'zeo::{0}'.format(name.lower()) in manager:
        event.form.widgets.errors += (Invalid(_("Specified connection name is already used!")), )
    connection = query_utility(IZEOConnection, name=name)
    if connection is not None:
        event.form.widgets.errors += (Invalid(_("A ZEO connection is already registered with this name!")), )


@view_config(name='add-zeo-connection.json', context=ISite, request_type=IPyAMSLayer,
             permission='system.manage', renderer='json', xhr=True)
class ZEOConnectionAJAXAddForm(AJAXAddForm, ZEOConnectionAddForm):
    """ZEO connection add form, AJAX view"""


@pagelet_config(name='properties.html', context=IZEOConnection, layer=IPyAMSLayer, permission='system.view')
class ZEOConnectionPropertiesEditForm(AdminDialogEditForm):
    """ZEO connection properties edit form"""

    @property
    def title(self):
        translate = self.request.localizer.translate
        return translate(_("ZEO connection: {0}")).format(self.context.name)

    legend = _("Update ZEO connection properties")
    icon_css_class = 'fa fa-fw fa-database'

    fields = field.Fields(IZEOConnection)
    autocomplete = 'off'
    ajax_handler = 'properties.json'
    edit_permission = 'system.manage'

    def updateWidgets(self, prefix=None):
        super(ZEOConnectionPropertiesEditForm, self).updateWidgets(prefix)
        self.widgets['name'].mode = DISPLAY_MODE


@view_config(name='properties.json', context=IZEOConnection, request_type=IPyAMSLayer,
             permission='system.manage', renderer='json', xhr=True)
class ZEOConnectionPropertiesAJAXEditForm(AJAXEditForm, ZEOConnectionPropertiesEditForm):
    """ZEO connection properties edit form, AJAX view"""
